@@ -19,12 +19,16 @@ our $Make_URI;
 
 our $SCHEME_RE = '[a-zA-Z][a-zA-Z0-9.+\-]*';
 
-our $VERSION = '0.021';
+our $VERSION = '0.50';
 
 # Class data
 use Rose::Class::MakeMethods::Generic
 (
-  inheritable_scalar => 'default_query_param_separator',
+  inheritable_scalar => 
+  [
+    'default_query_param_separator',
+    'default_omit_empty_query_params',
+  ],
 );
 
 # Object data
@@ -40,10 +44,11 @@ use Rose::Object::MakeMethods::Generic
     'path',
     'fragment',
     'query_param_separator' => { interface => 'get_set_init' },
-  ]
+  ],
 );
 
 __PACKAGE__->default_query_param_separator('&');
+__PACKAGE__->default_omit_empty_query_params(0);
 
 sub init_query_param_separator { ref(shift)->default_query_param_separator }
 
@@ -101,6 +106,8 @@ sub parse_query
 {
   my($self, $query) = @_;
 
+  $self->{'query_string'} = undef;
+
   unless(defined $query && $query =~ /\S/)
   {
     $self->{'query'} = { };
@@ -119,8 +126,8 @@ sub parse_query
   }
   elsif(index($query, '=') < 0)
   {
-    # XXX: Should be "keywords"?
-    $self->{'query'} = { $query => undef };
+    $self->{'query_string'} = __unescape_uri($query);
+    $self->{'query'} = { $self->{'query_string'} => undef };
     return 1;
   }
 
@@ -163,6 +170,19 @@ sub query_hash
   return (wantarray) ? %{$self->{'query'}} : { %{$self->{'query'}} };
 }
 
+sub omit_empty_query_params
+{
+  my($self) = shift;
+
+  if(@_)
+  {
+    return $self->{'omit_empty_query_params'} = $_[0] ? 1 : 0;
+  }
+
+  return defined $self->{'omit_empty_query_params'} ?
+    $self->{'omit_empty_query_params'} : ref($self)->default_omit_empty_query_params;
+}
+
 sub query_param
 {
   my($self) = shift;
@@ -174,6 +194,8 @@ sub query_param
   }
   elsif(@_ == 2)
   {
+    $self->{'query_string'} = undef;
+
     if(ref $_[1])
     {
       return $self->{'query'}{$_[0]} = [ @{$_[1]} ];
@@ -231,6 +253,11 @@ sub query_param_delete
 
   foreach my $param (@_)
   {
+    if(defined $self->{'query_string'} && $param eq $self->{'query_string'})
+    {
+      $self->{'query_string'} = undef;
+    }
+
     delete $self->{'query'}{$param};
   }
 }
@@ -274,13 +301,29 @@ sub query
     $self->{'query'} = _deep_copy({ @_ });
   }
 
-  return  unless(defined(wantarray));
+  my $want = wantarray;
 
-  my @query;
+  return  unless(defined wantarray);
+
+  if(defined $self->{'query_string'})
+  {
+    return __escape_uri($self->{'query_string'});
+  }
+
+  my(@query, $omit_empty);
 
   foreach my $param (sort keys %{$self->{'query'}})
   {
-    foreach my $value ($self->query_params($param))
+    my @values = $self->query_params($param);
+
+    # Contortions to avoid calling this method in the common(?) case where
+    # every query parameter has at least one value.
+    if(!@values && !(defined $omit_empty ? $omit_empty : ($omit_empty = $self->omit_empty_query_params)))
+    {
+      @values = ('');
+    }
+
+    foreach my $value (@values)
     {
       push(@query, __escape_uri($param) . '=' . __escape_uri($value));
     }
@@ -295,6 +338,7 @@ sub query_form
 
   if(@_)
   {
+    $self->{'query_string'} = undef;
     $self->{'query'} = { };
 
     for(my $i = 0; $i < $#_; $i += 2)
@@ -510,7 +554,7 @@ __END__
 
 =head1 NAME
 
-Rose::URI - A standalone URI class allowing easy and efficient manipulation of query parameters and other URI components.
+Rose::URI - A URI class that allows easy and efficient manipulation of URI components.
 
 =head1 SYNOPSIS
 
@@ -540,7 +584,7 @@ L<Rose::URI> is an alternative to L<URI>.  The important differences are as foll
 
 L<Rose::URI> provides a rich set of query string manipulation methods. Query parameters can be added, removed, and checked for their existence. L<URI> allows the entire query to be set or returned as a whole via the L<query_form|URI/query_form> or L<query|URI/query> methods, and the L<URI::QueryParam> module provides a few more methods for query string manipulation.
 
-L<Rose::URI> supports query parameters with multiple values (e.g. "a=1&a=2"). L<URI> has  limited support for this (through L<query_form|URI/query_form>'s list return value.  Better methods are available in L<URI::QueryParam>.
+L<Rose::URI> supports query parameters with multiple values (e.g. "a=1&a=2"). L<URI> has  limited support for this through L<query_form|URI/query_form>'s list return value.  Better methods are available in L<URI::QueryParam>.
 
 L<Rose::URI> uses Apache's C-based URI parsing and HTML escaping functions when running in a mod_perl 1.x web server environment.
 
@@ -595,6 +639,10 @@ All the above parameters accept strings.  See below for more information about t
 
 =over 4
 
+=item B<default_omit_empty_query_params [BOOL]>
+
+Get or set a boolean value that determines whether or not query parameters with "empty" (that is, undef or zero-length) values will be omitted from the L<query|/query> string by default.  The default value is false.
+
 =item B<default_query_param_separator [CHARACTER]>
 
 Get or set the character used to separate query parameters in the stringified version of L<Rose::URI> objects.  Defaults to "&".
@@ -623,6 +671,10 @@ Returns a copy of the L<Rose::URI> object.
 
 Get or set the fragment portion of the URI.
 
+=item B<omit_empty_query_params [BOOL]>
+
+Get or set a boolean value that determines whether or not query parameters with "empty" (that is, undef or zero-length) values will be omitted from the L<query|/query> string.  The default value is determined by the L<default_query_param_separator|/default_query_param_separator> class method.
+
 =item B<password [PASSWORD]>
 
 Get or set the password portion of the URI.
@@ -637,7 +689,7 @@ Get or set the port number portion of the URI.
 
 =item B<query [QUERY]>
 
-Get or sets the URI's query.  QUERY may be a query string (e.g. "a=1&b=2"), a reference to a hash, or a list of name/value pairs.
+Get or sets the URI's query.  QUERY may be an appropriately escaped query string (e.g. "a=1&b=2&c=a+long+string"), a reference to a hash, or a list of name/value pairs.
 
 Query strings may use either "&" or ";" as their query separator. If a "&" character exists anywhere in the query string, it is assumed to be the separator.
 
@@ -748,8 +800,8 @@ Get or set the username portion of the URI.
 
 =head1 AUTHOR
 
-John C. Siracusa (siracusa@mindspring.com)
+John C. Siracusa (siracusa@gmail.com)
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005 by John C. Siracusa.  All rights reserved.  This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+Copyright (c) 2008 by John C. Siracusa.  All rights reserved.  This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
